@@ -1,22 +1,30 @@
 """
-可自动实时调节线程数量的线程池。
-比官方ThreadpoolExecutor的改进是
-1.有界队列
-2.实时调节线程数量，指的是当任务很少时候会去关闭很多线程。官方ThreadpoolExecurot只能做到忙时候开启很多线程，但不忙时候线程没有关闭线程，
-此线程池实现了java ThreadpoolExecutor线程池的keppaliveTime参数的功能，linux系统能承受的线程总数有限，一般不到2万。
-3.能非常智能节制的开启多线程。比如设置线程池大小为500，线程池的运行函数消耗时间是只需要0.1秒，如果每隔2秒钟来一个任务。1个线程足够了，官方线程池是一直增长到500，然后不增长，官方的太不智能了。
+史上最强的python线程池。
 
-这个线程池是框架的默认线程方式的线程池，如果不设置并发方式就用的这里。
+最智能的可自动实时调节线程数量的线程池。此线程池和官方concurrent.futures的线程池 是鸭子类关系，所以可以一键替换类名 或者 import as来替换类名。
+对比官方线程池，有4个创新功能或改进。
+
+1、主要是不仅能扩大，还可自动缩小(官方内置的ThreadpoolExecutor不具备此功能，此概念是什么意思和目的，可以百度java ThreadpoolExecutor的KeepAliveTime参数的介绍)，
+
+2、非常节制的开启多线程，例如实例化一个最大100线程数目的pool，每隔2秒submit一个函数任务，而函数每次只需要1秒就能完成，实际上只需要调节增加到1个线程就可以，不需要慢慢增加到100个线程
+官方的线程池不够智能，会一直增加到最大线程数目，此线程池则不会。
+
+3、线程池任务的queue队列，修改为有界队列
+
+4、此线程池运行函数出错时候，直接显示线程错误，官方的线程池则不会显示错误，例如函数中写1/0,任然不现实错误。
+
+此实现了submit，还实现future相关的内容，真正的和内置的ThreadpoolExecutor 完全替代。
 """
-
+import os
 import atexit
 import queue
 import sys
 import threading
 import time
 import weakref
-
 from nb_log import LoggerMixin, nb_print, LoggerLevelSetterMixin, LogManager
+from concurrent.futures import Executor, Future
+
 from function_scheduling_distributed_framework.concurrent_pool.custom_evenlet_pool_executor import check_evenlet_monkey_patch
 from function_scheduling_distributed_framework.concurrent_pool.custom_gevent_pool_executor import check_gevent_monkey_patch
 
@@ -24,6 +32,13 @@ _shutdown = False
 _threads_queues = weakref.WeakKeyDictionary()
 
 
+def check_not_monkey():
+    if check_gevent_monkey_patch(raise_exc=False):
+        raise Exception('请不要打gevent包的补丁')
+    if check_evenlet_monkey_patch(raise_exc=False):
+        raise Exception('请不要打evenlet包的补丁')
+
+
 def _python_exit():
     global _shutdown
     _shutdown = True
@@ -38,30 +53,35 @@ atexit.register(_python_exit)
 
 
 class _WorkItem(LoggerMixin):
-    def __init__(self, fn, args, kwargs):
+    def __init__(self, future, fn, args, kwargs):
+        self.future = future
         self.fn = fn
         self.args = args
         self.kwargs = kwargs
 
     def run(self):
         # noinspection PyBroadException
+        if not self.future.set_running_or_notify_cancel():
+            return
         try:
-            self.fn(*self.args, **self.kwargs)
+            result = self.fn(*self.args, **self.kwargs)
         except BaseException as exc:
             self.logger.exception(f'函数 {self.fn.__name__} 中发生错误，错误原因是 {type(exc)} {exc} ')
+            self.future.set_exception(exc)
+            # Break a reference cycle with the exception 'exc'
+            self = None  # noqa
+        else:
+            self.future.set_result(result)
 
     def __str__(self):
         return f'{(self.fn.__name__, self.args, self.kwargs)}'
 
 
-def check_not_monkey():
-    if check_gevent_monkey_patch(raise_exc=False):
-        raise Exception('请不要打gevent包的补丁')
-    if check_evenlet_monkey_patch(raise_exc=False):
-        raise Exception('请不要打evenlet包的补丁')
+class ThreadPoolExecutorShrinkAble(Executor, LoggerMixin, LoggerLevelSetterMixin):
+    # 为了和官方自带的THredpoolexecutor保持完全一致的鸭子类，参数设置成死的，不然用户传参了。
+    MIN_WORKERS = 5
+    KEEP_ALIVE_TIME = 60
 
-
-class CustomThreadPoolExecutor(LoggerMixin, LoggerLevelSetterMixin):
     def __init__(self, max_workers=None, thread_name_prefix=''):
         """
         最好需要兼容官方concurren.futures.ThreadPoolExecutor 和改版的BoundedThreadPoolExecutor，入参名字和个数保持了一致。
@@ -69,22 +89,16 @@ class CustomThreadPoolExecutor(LoggerMixin, LoggerLevelSetterMixin):
         :param thread_name_prefix:
         """
         self._max_workers = max_workers or 4
-        self._min_workers = 5 # 这是对应的 java Threadpoolexecutor的corePoolSize，为了保持线程池公有方法和与py官方内置的concurren.futures.ThreadPoolExecutor一致，不增加更多的实例化时候入参，这里写死为5.
         self._thread_name_prefix = thread_name_prefix
-        self.work_queue = queue.Queue(max_workers)
+        self.work_queue = self._work_queue = queue.Queue(max_workers)
         # self._threads = set()
         self._threads = weakref.WeakSet()
         self._lock_compute_threads_free_count = threading.Lock()
         self.threads_free_count = 0
         self._shutdown = False
         self._shutdown_lock = threading.Lock()
-        # self.logger.setLevel(20)
 
-    def set_min_workers(self, min_workers=10):
-        self._min_workers = min_workers
-        return self
-
-    def change_threads_free_count(self, change_num):
+    def _change_threads_free_count(self, change_num):
         with self._lock_compute_threads_free_count:
             self.threads_free_count += change_num
 
@@ -92,22 +106,22 @@ class CustomThreadPoolExecutor(LoggerMixin, LoggerLevelSetterMixin):
         with self._shutdown_lock:
             if self._shutdown:
                 raise RuntimeError('不能添加新的任务到线程池')
-        self.work_queue.put(_WorkItem(func, args, kwargs))
-        self._adjust_thread_count()
+            f = Future()
+            w = _WorkItem(f, func, args, kwargs)
+            self.work_queue.put(w)
+            self._adjust_thread_count()
+            return f
 
     def _adjust_thread_count(self):
-        # if len(self._threads) < self._threads_num:
-        # self.logger.debug((self.threads_free_count, len(self._threads), len(_threads_queues), get_current_threads_num()))
-        if self.threads_free_count < self._min_workers and len(self._threads) < self._max_workers:
-            # t = threading.Thread(target=_work,
-            #                      args=(self._work_queue,self))
+        # print(self.threads_free_count, self.MIN_WORKERS, len(self._threads), self._max_workers)
+        if self.threads_free_count < self.MIN_WORKERS and len(self._threads) < self._max_workers:
             t = _CustomThread(self).set_log_level(self.logger.level)
-            t.setDaemon(True)
+            t.daemon = True
             t.start()
             self._threads.add(t)
-            _threads_queues[t] = self.work_queue
+            _threads_queues[t] = self._work_queue
 
-    def shutdown(self, wait=True):
+    def shutdown(self, wait=True):  # noqa
         with self._shutdown_lock:
             self._shutdown = True
             self.work_queue.put(None)
@@ -115,25 +129,23 @@ class CustomThreadPoolExecutor(LoggerMixin, LoggerLevelSetterMixin):
             for t in self._threads:
                 t.join()
 
-    def __enter__(self):
-        return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        self.shutdown(wait=True)
-        return False
+# 两个名字都可以，兼容以前的老名字（中文意思是 自定义线程池），但新名字更能表达意思（可缩小线程池）。
+CustomThreadpoolExecutor = CustomThreadPoolExecutor = ThreadPoolExecutorShrinkAble
 
 
+# noinspection PyProtectedMember
 class _CustomThread(threading.Thread, LoggerMixin, LoggerLevelSetterMixin):
-    def __init__(self, executorx: CustomThreadPoolExecutor):
+    _lock_for_judge_threads_free_count = threading.Lock()
+
+    def __init__(self, executorx: ThreadPoolExecutorShrinkAble):
         super().__init__()
         self._executorx = executorx
-        self._run_times = 0
 
-    # noinspection PyProtectedMember
     def _remove_thread(self, stop_resson=''):
         # noinspection PyUnresolvedReferences
         self.logger.debug(f'停止线程 {self._ident}, 触发条件是 {stop_resson} ')
-        self._executorx.change_threads_free_count(-1)
+        self._executorx._change_threads_free_count(-1)
         self._executorx._threads.remove(self)
         _threads_queues.pop(self)
 
@@ -141,26 +153,27 @@ class _CustomThread(threading.Thread, LoggerMixin, LoggerLevelSetterMixin):
     def run(self):
         # noinspection PyUnresolvedReferences
         self.logger.debug(f'新启动线程 {self._ident} ')
-        self._executorx.change_threads_free_count(1)
+        self._executorx._change_threads_free_count(1)
         while True:
             try:
-                work_item = self._executorx.work_queue.get(block=True, timeout=60)
+                work_item = self._executorx.work_queue.get(block=True, timeout=self._executorx.KEEP_ALIVE_TIME)
             except queue.Empty:
                 # continue
                 # self._remove_thread()
-                # break
-                if self._executorx.threads_free_count > self._executorx._min_workers:
-                    self._remove_thread(f'当前线程超过60秒没有任务，线程池中不在工作状态中的线程数量是 {self._executorx.threads_free_count}，超过了指定的数量 {self._executorx._min_workers}')
-                    break   # 退出while 1，即是结束。这里才是决定线程结束销毁，_remove_thread只是个名字而已，不是由那个来销毁线程。
-                else:
-                    continue
+                with self._lock_for_judge_threads_free_count:
+                    if self._executorx.threads_free_count > self._executorx.MIN_WORKERS:
+                        self._remove_thread(
+                            f'当前线程超过 {self._executorx.KEEP_ALIVE_TIME} 秒没有任务，线程池中不在工作状态中的线程数量是 '
+                            f'{self._executorx.threads_free_count}，超过了指定的数量 {self._executorx.MIN_WORKERS}')
+                        break  # 退出while 1，即是结束。这里才是决定线程结束销毁，_remove_thread只是个名字而已，不是由那个来销毁线程。
+                    else:
+                        continue
 
-            # nb_print(work_item)
             if work_item is not None:
-                self._executorx.change_threads_free_count(-1)
+                self._executorx._change_threads_free_count(-1)
                 work_item.run()
                 del work_item
-                self._executorx.change_threads_free_count(1)
+                self._executorx._change_threads_free_count(1)
                 continue
             if _shutdown or self._executorx._shutdown:
                 self._executorx.work_queue.put(None)
@@ -168,23 +181,26 @@ class _CustomThread(threading.Thread, LoggerMixin, LoggerLevelSetterMixin):
 
 
 process_name_set = set()
-logger_show_current_threads_num = LogManager('show_current_threads_num').get_logger_and_add_handlers(formatter_template=5, log_filename='show_current_threads_num.log', do_not_use_color_handler=True)
+logger_show_current_threads_num = LogManager('show_current_threads_num').get_logger_and_add_handlers(
+    formatter_template=5, log_filename='show_current_threads_num.log', do_not_use_color_handler=False)
 
 
-def show_current_threads_num(sleep_time=60, process_name='', block=False):
+def show_current_threads_num(sleep_time=600, process_name='', block=False, daemon=True):
     process_name = sys.argv[0] if process_name == '' else process_name
 
     def _show_current_threads_num():
         while True:
             # logger_show_current_threads_num.info(f'{process_name} 进程 的 并发数量是 -->  {threading.active_count()}')
-            nb_print(f'{process_name} 进程 的 线程数量是 -->  {threading.active_count()}')
+            # nb_print(f'  {process_name} {os.getpid()} 进程 的 线程数量是 -->  {threading.active_count()}')
+            logger_show_current_threads_num.info(
+                f'  {process_name} {os.getpid()} 进程 的 线程数量是 -->  {threading.active_count()}')
             time.sleep(sleep_time)
 
     if process_name not in process_name_set:
         if block:
             _show_current_threads_num()
         else:
-            t = threading.Thread(target=_show_current_threads_num, daemon=True)
+            t = threading.Thread(target=_show_current_threads_num, daemon=daemon)
             t.start()
         process_name_set.add(process_name)
 
@@ -194,29 +210,25 @@ def get_current_threads_num():
 
 
 if __name__ == '__main__':
-    from function_scheduling_distributed_framework.utils import decorators
-    from function_scheduling_distributed_framework.concurrent_pool.bounded_threadpoolexcutor import BoundedThreadPoolExecutor
-
-
-    # @decorators.keep_circulating(1)
-    def f1(a):
-        time.sleep(0.2)
-        nb_print(f'{a} 。。。。。。。')
-        # raise Exception('抛个错误测试')
-
-
-    # show_current_threads_num()
-    pool = CustomThreadPoolExecutor(200).set_log_level(10).set_min_workers()
-    # pool = BoundedThreadPoolExecutor(200)   # 测试对比原来写的BoundedThreadPoolExecutor
     show_current_threads_num(sleep_time=5)
+
+
+    def f1(a):
+        time.sleep(2)  # 可修改这个数字测试多线程数量调节功能。
+        nb_print(f'{a} 。。。。。。。')
+        return a * 10
+        # raise Exception('抛个错误测试')  # 官方的不会显示函数出错你，你还以为你写的代码没毛病呢。
+
+
+    pool = ThreadPoolExecutorShrinkAble(200)
+    # pool = ThreadPoolExecutor(200)  # 测试对比官方自带
+
     for i in range(300):
-        time.sleep(0.3)  # 这里的间隔时间模拟，当任务来临不密集，只需要少量线程就能搞定f1了，因为f1的消耗时间短，不需要开那么多线程，CustomThreadPoolExecutor比BoundedThreadPoolExecutor 优势之一。
-        pool.submit(f1, str(i))
+        time.sleep(0.5)  # 这里的间隔时间模拟，当任务来临不密集，只需要少量线程就能搞定f1了，因为f1的消耗时间短，
+        # 不需要开那么多线程，CustomThreadPoolExecutor比ThreadPoolExecutor 优势之一。
+        futurex = pool.submit(f1, i)
+        # print(futurex.result())
 
-    nb_print(6666)
-    # pool.shutdown(wait=True)
-    pool.submit(f1, 'yyyy')
-
-    # 下面测试阻塞主线程退出的情况。注释掉可以测主线程退出的情况。
-    while True:
-        time.sleep(10)
+    # 1/下面测试阻塞主线程退出的情况。注释掉可以测主线程退出的情况。
+    # 2/此代码可以证明，在一段时间后，连续长时间没任务，官方线程池的线程数目还是保持在最大数量了。而此线程池会自动缩小，实现了java线程池的keppalivetime功能。
+    time.sleep(1000000)
