@@ -3,15 +3,14 @@
 # @Time    : 2019/8/8 0008 13:11
 """
 所有中间件类型消费者的抽象基类。使实现不同中间件的消费者尽可能代码少。
+真个流程最难的都在这里面。
 """
-from typing import List
 
 import abc
 import copy
 # from multiprocessing import Process
 import datetime
 import json
-import pymongo
 import sys
 import socket
 import os
@@ -25,13 +24,16 @@ from threading import Lock, Thread
 import eventlet
 import gevent
 import asyncio
+import pymongo
 from pymongo import IndexModel
 from pymongo.errors import PyMongoError
 
 # noinspection PyUnresolvedReferences
 from nb_log import LoggerLevelSetterMixin, LogManager, nb_print, LoggerMixin, LoggerMixinDefaultWithFileHandler
 # noinspection PyUnresolvedReferences
+from function_scheduling_distributed_framework.concurrent_pool.async_helper import simple_run_in_executor
 from function_scheduling_distributed_framework.concurrent_pool.async_pool_executor import AsyncPoolExecutor
+# noinspection PyUnresolvedReferences
 from function_scheduling_distributed_framework.concurrent_pool.bounded_threadpoolexcutor import \
     BoundedThreadPoolExecutor
 from function_scheduling_distributed_framework.concurrent_pool.custom_evenlet_pool_executor import evenlet_timeout_deco, \
@@ -259,7 +261,7 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         """
         ConsumersManager.join_all_consumer_shedual_task_thread()
 
-    # noinspection PyProtectedMember
+    # noinspection PyProtectedMember,PyUnresolvedReferences
     def __init__(self, queue_name, *, consuming_function: Callable = None, function_timeout=0,
                  threads_num=50, concurrent_num=50, specify_concurrent_pool=None, specify_async_loop=None, concurrent_mode=1,
                  max_retry_times=3, log_level=10, is_print_detail_exception=True,
@@ -282,7 +284,7 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         :param concurrent_num:并发数量，这个覆盖threads_num。以后会废弃threads_num参数，因为表达的意思不太准确，不一定是线程模式并发。
         :param specify_concurrent_pool:使用指定的线程池/携程池，可以多个消费者共使用一个线程池，不为None时候。threads_num失效
         :param specify_async_loop:指定的async的loop循环，设置并发模式为async才能起作用。
-        :param concurrent_mode:并发模式，暂时支持 线程 、gevent、eventlet三种模式。  1线程  2 gevent 3 evenlet
+        :param concurrent_mode:并发模式，1线程 2gevent 3eventlet 4 asyncio
         :param max_retry_times:
         :param log_level:
         :param is_print_detail_exception:
@@ -320,13 +322,16 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
 
         # 方便点击跳转定位到当前解释器下所有实例化消费者的文件行，点击可跳转到该处。
         # 获取被调用函数在被调用时所处代码行数
-        # 直接实例化相应的类和使用工厂模式来实例化相应的类，得到的消费者实际实例化的行是不一样的，希望定位到用户的代码处，而不是定位到工厂模式处。
+        # 直接实例化相应的类和使用工厂模式来实例化相应的类，得到的消费者实际实例化的行是不一样的，希望定位到用户的代码处，而不是定位到工厂模式处。也不要是task_deco装饰器本身处。
         line = sys._getframe(0).f_back.f_lineno
         # 获取被调用函数所在模块文件名
         file_name = sys._getframe(1).f_code.co_filename
         if 'consumer_factory.py' in file_name:
             line = sys._getframe(1).f_back.f_lineno
             file_name = sys._getframe(2).f_code.co_filename
+        if r'function_scheduling_distributed_framework\__init__.py' in file_name or 'function_scheduling_distributed_framework/__init__.py' in file_name:
+            line = sys._getframe(2).f_back.f_lineno
+            file_name = sys._getframe(3).f_code.co_filename
         current_queue__info_dict['where_to_instantiate'] = f'{file_name}:{line}'
 
         self._queue_name = queue_name
@@ -537,14 +542,15 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
                 if asyncio.iscoroutine(function_result_status.result):
                     self.logger.critical(f'异步的协程消费函数必须使用 async 并发模式并发,请设置 '
                                          f'消费函数 {self.consuming_function.__name__} 的concurrent_mode 为4')
-                    # noinspection PyProtectedMember
+                    # noinspection PyProtectedMember,PyUnresolvedReferences
                     os._exit(4)
                 function_result_status.success = True
                 self._confirm_consume(kw)
                 if self.__get_priority_conf(kw, 'do_task_filtering'):
                     self._redis_filter.add_a_value(function_only_params)  # 函数执行成功后，添加函数的参数排序后的键值对字符串到set中。
                 self.logger.debug(f' 函数 {self.consuming_function.__name__}  '
-                                  f'第{current_retry_times + 1}次 运行, 正确了，函数运行时间是 {round(time.time() - t_start, 4)} 秒,入参是 【 {function_only_params} 】。  {self._get_concurrent_info()}')
+                                  f'第{current_retry_times + 1}次 运行, 正确了，函数运行时间是 {round(time.time() - t_start, 4)} 秒,入参是 【 {function_only_params} 】。 '
+                                  f' {self._get_concurrent_info()}')
             except Exception as e:
                 if isinstance(e, (PyMongoError,
                                   ExceptionForRequeue)):  # mongo经常维护备份时候插入不了或挂了，或者自己主动抛出一个ExceptionForRequeue类型的错误会重新入队，不受指定重试次数逇约束。
@@ -560,6 +566,9 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
             self.logger.critical(
                 f'函数 {self.consuming_function.__name__} 达到最大重试次数 {self.__get_priority_conf(kw, "max_retry_times")} 后,仍然失败， 入参是 【 {function_only_params} 】')
             self._confirm_consume(kw)  # 错得超过指定的次数了，就确认消费了。
+            if self.__get_priority_conf(kw, 'do_task_filtering'):
+                self._redis_filter.add_a_value(function_only_params)  # 函数执行成功后，添加函数的参数排序后的键值对字符串到set中。
+
         if self.__get_priority_conf(kw, 'is_using_rpc_mode'):
             # print(function_result_status.get_status_dict(without_datetime_obj=True))
             with RedisMixin().redis_db_frame.pipeline() as p:
@@ -572,12 +581,18 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         self._result_persistence_helper.save_function_result_to_mongo(function_result_status)
 
     async def _async_run(self, kw: dict, ):
+        """这个是为了asyncio模式的"""
         function_only_params = _delete_keys_and_return_new_dict(kw['body'], )
-        if self.__get_priority_conf(kw, 'do_task_filtering') and self._redis_filter.check_value_exists(
-                function_only_params):  # 对函数的参数进行检查，过滤已经执行过并且成功的任务。
-            self.logger.warning(f'redis的 [{self._redis_filter_key_name}] 键 中 过滤任务 {kw["body"]}')
-            self._confirm_consume(kw)
-            return
+        # if self.__get_priority_conf(kw, 'do_task_filtering') and self._redis_filter.check_value_exists(
+        #         function_only_params):  # 对函数的参数进行检查，过滤已经执行过并且成功的任务。
+        if self.__get_priority_conf(kw, 'do_task_filtering'):
+            is_exists = await simple_run_in_executor(self._redis_filter.check_value_exists,
+                                                     function_only_params)
+            if is_exists:
+                self.logger.warning(f'redis的 [{self._redis_filter_key_name}] 键 中 过滤任务 {kw["body"]}')
+                # self._confirm_consume(kw)
+                await simple_run_in_executor(self._confirm_consume, kw)
+                return
         t_start_run_fun = time.time()
         await self._async_run_consuming_function_with_confirm_and_retry(kw, current_retry_times=0,
                                                                         function_result_status=FunctionResultStatus(
@@ -608,7 +623,7 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
                 if not asyncio.iscoroutine(corotinue_obj):
                     self.logger.critical(f'当前设置的并发模式为 async 并发模式，但消费函数不是异步协程函数，'
                                          f'请不要把消费函数 {self.consuming_function.__name__} 的 concurrent_mode 设置为 4')
-                    # noinspection PyProtectedMember
+                    # noinspection PyProtectedMember,PyUnresolvedReferences
                     os._exit(444)
                 if self._function_timeout == 0:
                     rs = await corotinue_obj
@@ -617,17 +632,21 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
                     rs = await asyncio.wait_for(corotinue_obj, timeout=self._function_timeout)
                 function_result_status.result = rs
                 function_result_status.success = True
-                self._confirm_consume(kw)
+                # self._confirm_consume(kw)
+                await simple_run_in_executor(self._confirm_consume, kw)
                 if self.__get_priority_conf(kw, 'do_task_filtering'):
-                    self._redis_filter.add_a_value(function_only_params)  # 函数执行成功后，添加函数的参数排序后的键值对字符串到set中。
+                    # self._redis_filter.add_a_value(function_only_params)  # 函数执行成功后，添加函数的参数排序后的键值对字符串到set中。
+                    await simple_run_in_executor(self._redis_filter.add_a_value, function_only_params)
                 self.logger.debug(f' 函数 {self.consuming_function.__name__}  '
-                                  f'第{current_retry_times + 1}次 运行, 正确了，函数运行时间是 {round(time.time() - t_start, 4)} 秒,入参是 【 {function_only_params} 】。  {dir(corotinue_obj.cr_running)}')
+                                  f'第{current_retry_times + 1}次 运行, 正确了，函数运行时间是 {round(time.time() - t_start, 4)} 秒,入参是 【 {function_only_params} 】。  {corotinue_obj}')
             except Exception as e:
                 if isinstance(e, (PyMongoError,
                                   ExceptionForRequeue)):  # mongo经常维护备份时候插入不了或挂了，或者自己主动抛出一个ExceptionForRequeue类型的错误会重新入队，不受指定重试次数逇约束。
                     self.logger.critical(f'函数 [{self.consuming_function.__name__}] 中发生错误 {type(e)}  {e}，消息重新入队')
-                    time.sleep(1)  # 防止快速无限出错入队出队，导致cpu和中间件忙
-                    return self._requeue(kw)
+                    # time.sleep(1)  # 防止快速无限出错入队出队，导致cpu和中间件忙
+                    await asyncio.sleep(1)
+                    # return self._requeue(kw)
+                    return await simple_run_in_executor(self._requeue, kw)
                 self.logger.error(f'函数 {self.consuming_function.__name__}  第{current_retry_times + 1}次发生错误，'
                                   f'函数运行时间是 {round(time.time() - t_start, 4)} 秒,\n  入参是 【 {function_only_params} 】   \n 原因是 {type(e)} {e} ',
                                   exc_info=self.__get_priority_conf(kw, 'is_print_detail_exception'))
@@ -636,17 +655,23 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         else:
             self.logger.critical(
                 f'函数 {self.consuming_function.__name__} 达到最大重试次数 {self.__get_priority_conf(kw, "max_retry_times")} 后,仍然失败， 入参是 【 {function_only_params} 】')
-            self._confirm_consume(kw)  # 错得超过指定的次数了，就确认消费了。
+            # self._confirm_consume(kw)  # 错得超过指定的次数了，就确认消费了。
+            await simple_run_in_executor(self._confirm_consume, kw)
+            if self.__get_priority_conf(kw, 'do_task_filtering'):
+                # self._redis_filter.add_a_value(function_only_params)  # 函数执行成功后，添加函数的参数排序后的键值对字符串到set中。
+                await simple_run_in_executor(self._redis_filter.add_a_value, function_only_params)
+
         if self.__get_priority_conf(kw, 'is_using_rpc_mode'):
-            # print(function_result_status.get_status_dict(without_datetime_obj=True))
-            with RedisMixin().redis_db_frame.pipeline() as p:
-                # RedisMixin().redis_db_frame.lpush(kw['body']['extra']['task_id'], json.dumps(function_result_status.get_status_dict(without_datetime_obj=True)))
-                # RedisMixin().redis_db_frame.expire(kw['body']['extra']['task_id'], 600)
-                p.lpush(kw['body']['extra']['task_id'],
-                        json.dumps(function_result_status.get_status_dict(without_datetime_obj=True)))
-                p.expire(kw['body']['extra']['task_id'], 600)
-                p.execute()
-        self._result_persistence_helper.save_function_result_to_mongo(function_result_status)
+            def push_result():
+                with RedisMixin().redis_db_frame.pipeline() as p:
+                    p.lpush(kw['body']['extra']['task_id'],
+                            json.dumps(function_result_status.get_status_dict(without_datetime_obj=True)))
+                    p.expire(kw['body']['extra']['task_id'], 600)
+                    p.execute()
+
+            await simple_run_in_executor(push_result)
+        # self._result_persistence_helper.save_function_result_to_mongo(function_result_status)
+        await simple_run_in_executor(self._result_persistence_helper.save_function_result_to_mongo, function_result_status)
 
     @abc.abstractmethod
     def _confirm_consume(self, kw):
@@ -796,6 +821,7 @@ class ConcurrentModeDispatcher(LoggerMixin):
                 t.start()
 
 
+# noinspection DuplicatedCode
 def wait_for_possible_has_finish_all_tasks(queue_name: str, minutes: int, send_stop_to_broker=0,
                                            broker_kind: int = 0, ):
     """
